'' =================================================================================================
''
''   File....... jm_strings.spin
''   Purpose.... Miscellaneous string methods
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2011-2015 Jon McPhalen
''               -- see below for terms of use
''   E-mail.....
''   Started....
''   Updated.... 03 FEB 2015
''
'' =================================================================================================


pub ucstr(p_str)

'' Converts z-string at pntr to upper case

  repeat strsize(p_str)
    byte[p_str] := upper(byte[p_str])
    p_str++


pub upper(c)

'' Convert c to uppercase
'' -- does not modify non-alphas

  if ((c => "a") and (c =< "z"))
    c -= 32

  return c


pub lcstr(p_str)

'' Converts z-string at pntr to lower case

  repeat strsize(p_str)
    byte[p_str] := lower(byte[p_str])
    p_str++


pub lower(c)

'' Convert c to lowercase
'' -- does not modify non-alphas

  if ((c => "A") and (c =< "Z"))
    c += 32

  return c


pub is_alpha(c)

'' Returns true if c is alpha character

  if ((c => "a") and (c =< "z"))
    return true
  elseif ((c => "A") and (c =< "Z"))
    return true
  else
    return false


pub is_digit(c)

'' Returns true if c is digit character

  if ((c => "0") and (c =< "9"))
    return true
  else
    return false


pub is_alphanum(c)

'' Returns true if c is alpha/numeric character

  if ((c => "a") and (c =< "z"))
    return true
  elseif ((c => "A") and (c =< "Z"))
    return true
  elseif ((c => "0") and (c =< "9"))
    return true
  else
    return false


pub is_decimal(p_str) | c

'' Returns true if string is decimal format
'' -- separator okay

  repeat strsize(p_str)
    c := byte[p_str++]
    ifnot (is_digit(c))
      ifnot (c == "_")
        return false

  return true


pub is_binary(p_str) | c

'' Returns true if string is binary format
'' -- excludes format indicator
'' -- separator okay

  repeat strsize(p_str)
    c := byte[p_str++]
    ifnot ((c => "0") and (c =< "1"))
      ifnot (c == "_")
        return false

  return true


pub is_hex(p_str) | c

'' Returns true if string is hexadecimal format
'' -- excludes format indicator

  repeat strsize(p_str)
    c := upper(byte[p_str++])
    ifnot (is_digit(c))
      ifnot ((c => "A") and (c =< "Z"))
        ifnot (c == "_")
          return false

  return true


pub is_number(p_str)

'' Returns true if string is number in known format (dec, bin, or hex)

  if (byte[p_str] == "$")                                        ' hex indicator?
    return is_hex(p_str+1)

  if (byte[p_str] == "%")                                        ' binary indicator?
    return is_binary(p_str+1)

  if lookdown(byte[p_str] : "+-", "0".."9")                      ' sign or digit?
    return is_decimal(p_str+1)

  return false


pub strncmp(p_str1, p_str2, n) | match

'' Compares n characters of str2 with str1
'' -- p_str1 and p_str2 are pointers to strings (or byte arrays)
'' -- 0 if str1 == str2, 1 if str1 > str2, -1 if str1 < str2

  match := 0

  if (n > 0)
    repeat n
      if (byte[p_str1] > byte[p_str2])
        ++match
        quit
      elseif (byte[p_str1] < byte[p_str2])
        --match
        quit
      else
        ++p_str1
        ++p_str2

  return match


pub instr(p_str1, p_str2) | len1, len2, pos, idx

'' Returns position of str2 in str1
'' -- p_str1 is pointer to z-string to search
'' -- p_str2 is pointer to z-string to look for in str1
'' -- if str2 not in str1 returns -1

  len1 := strsize(p_str1)
  len2 := strsize(p_str2)
  pos  := -1
  idx  := 0

  if (len1 >= len2)
    repeat (len1 - len2 + 1)
      if (byte[p_str1] == 0)
        quit
      else
        if (strncmp(p_str1++, p_str2, len2) == 0)
          pos := idx
          quit
        else
          ++idx

  return pos


pub cinstr(p_str, c) | pos

'' Returns position of character c in string at p_str
'' -- returns -1 if not found

  pos := 0

  repeat strsize(p_str)                                          ' loop through string chars
    if (byte[p_str][pos] == c)                                   ' if match
      return pos                                                 ' return position
    pos += 1

  return -1                                                      ' return not found


pub strncopy(p_str1, p_str2, n) | c

'' Copies n characters from str2 to str1
'' -- str1 and str2 are pointers to strings

  repeat n
    c := byte[p_str2++]
    byte[p_str1++] := c
    if (c == 0)
      quit


pub left(p_dest, p_src, len)

'' Copies left len characters from src string to dest string
'' -- dest and src are pointers to string buffers

  bytemove(p_dest, p_src, len)
  byte[p_dest][len] := 0


pub right(p_dest, p_src, len) | sl

'' Copies right len characters from src string to dest string
'' -- dest and src are pointers to string buffers

  sl := strsize(p_src)                                           ' length of source
  bytemove(p_dest, p_src+(sl-len), len+1)                        ' include com1inating 0


pub mid(p_dest, p_src, start, len)

'' Copies middle len characters from src string to dest string
'' -- dest and src are pointers to string buffers
'' -- start is zero indexed

  bytemove(p_dest, p_src+start, len)
  byte[p_dest][len] := 0


con

  { --------------------- }
  {  Numeric conversions  }
  { --------------------- }


pub asc2val(p_str) | c

'' Returns value of numeric string
'' -- binary (%) and hex ($) must be indicated

  repeat
    c := byte[p_str]
    case c
      " ":                                                       ' skip leading space(s)
        p_str++

      "+", "-", "0".."9":                                        ' found decimal value
        return asc2dec(p_str, 11)

      "%":                                                       ' found binary value
        return bin2dec(p_str, 32)

      "$":                                                       ' found hex value
        return hex2dec(p_str, 8)

      other:                                                     ' abort on bad character
        return 0


pub asc2dec(p_str, n) | c, value, sign

'' Returns signed value from decimal string
'' -- p_str is pointer to decimal string
'' -- n is maximum number of digits to process

  if (n < 1)                                                     ' if bogus, bail out
    return 0

  value := 0                                                     ' initialize value
  sign := 1                                                      ' assume positive

  ' trim leading chars (spaces / sign)

  repeat
    c := byte[p_str]
    case c
      " ":                                                       ' skip leading space(s)
        p_str++

      "0".."9":                                                  ' found #s, extract value
        quit

      "+":
        p_str++                                                  ' skip sign, extract value
        quit

      "-":
        sign := -1                                               ' value is negative
        p_str++                                                  ' skip sign, extract value
        quit

      other:                                                     ' abort on bad character
        return 0

  ' extract numeric character
  ' -- can contain comma and underscore separators

  n <#= 10                                                       ' limit to 10 digits

  repeat while (n > 0)
    c := byte[p_str++]
    case c
      "0".."9":                                                  ' digit?
        value := (value * 10) + (c - "0")                        '  update value
        n--

      "_":
        { ignore }

      other:
        quit

  return sign * value


pub bin2dec(p_str, n) | c, value

'' Returns value from {indicated} binary string
'' -- pntr is pointer to binary string
'' -- n is maximum number of digits to process

  if (n < 1)                                                     ' if bogus, bail out
    return 0

  repeat
    c := byte[p_str]
    case c
      " ":                                                       ' skip leading space(s)
        p_str++

      "%":                                                       ' found indicator
        p_str++                                                  '  move to value
        quit

      "0".."1":                                                  ' found value
        quit

      other:                                                     ' abort on bad character
        return 0

  value := 0

  n <#= 32                                                       ' limit chars in value

  repeat while (n)
    c := byte[p_str++]                                           ' get next character
    case c
      "0".."1":                                                  ' binary digit?
        value := (value << 1) | (c - "0")                        '  update value
        --n                                                      '  dec digits count

      "_":
        ' skip

      other:
        quit

  return value


pub hex2dec(p_str, n) | c, value

'' Returns value from {indicated} hex string
'' -- p_str is pointer to binary string
'' -- n is maximum number of digits to process

  if (n < 1)                                                     ' if bogus, bail out
    return 0

  repeat
    c := upper(byte[p_str])
    case c
      " ":                                                       ' skip leading space(s)
        p_str++

      "$":                                                       ' found indicator
        p_str++                                                  '  move to value
        quit

      "0".."9", "A".."F":                                        ' found value
        quit

      other:                                                     ' abort on bad character
        return 0

  value := 0

  n <#= 8                                                        ' limit field width

  repeat while (n)
    c := upper(byte[p_str++])
    case c
      "0".."9":                                                  ' digit?
        value := (value << 4) | (c - "0")                        '  update value
        --n                                                      '  dec digits count

      "A".."F":                                                  ' hex digit?
        value := (value << 4) | (c - "A" + 10)
        --n

      "_":
        { skip }

      other:
        quit

  return value


dat { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}